@@ -5,13 +5,14 @@ import json
 from pymongo.collation import Collation, CollationStrength
 import pymongo
 
+
 class QueryTaxonTree(query_nosql.DataQuery):
     '''Queries specific to taxon_tree collection
     '''
 
     def __init__(self, cache_dirname=None, collection_str='taxon_tree', 
                 verbose=False, max_entries=float('inf'), username=None, MongoDB=None, 
-                password=None, db='datanator', authSource='admin', readPreference='primary'):
+                password=None, db='datanator', authSource='admin', readPreference='nearest'):
         self.collection_str = collection_str
         super(query_nosql.DataQuery, self).__init__(cache_dirname=cache_dirname, MongoDB=MongoDB,
                                         db=db, verbose=verbose, max_entries=max_entries, username=username,
@@ -56,18 +57,19 @@ class QueryTaxonTree(query_nosql.DataQuery):
     def get_name_by_id(self, ids):
         ''' Get organisms' names given their tax_ids
             Args:
-                ids: organisms' tax_ids
+                ids (:obj:`list`): list of organisms' tax_ids
             Return:
-                names: organisms' names
+                (:obj:`dict`): organisms' ids and names
         '''
-        names = []
-        projection = {'_id': 0, 'tax_name': 1}
+        names = {}
+        projection = {'_id': 0, 'tax_name': 1, 'tax_id': 1}
         query = {'tax_id': {'$in': ids}}
         docs = self.collection.find(filter=query, projection=projection)
         if docs is None:
-            return ['not in database']
+            return {-1: 'none in database'}
         for doc in docs:
-            names.append(doc['tax_name'])
+            _id = doc['tax_id']
+            names[_id] = doc['tax_name']
         return names
 
     def get_anc_by_name(self, names):
@@ -82,10 +84,10 @@ class QueryTaxonTree(query_nosql.DataQuery):
         result_id = []
         result_name = []
         projection = {'_id': 0, 'anc_id': 1, 'anc_name': 1}
-        query = {'tax_name': {'$in': names}}
-        docs = self.collection.find(filter=query, collation=self.collation,
-                                    projection=projection)
-        for doc in docs:
+        for name in names:
+            query = {'tax_name': name}
+            doc = self.collection.find_one(filter=query, collation=self.collation,
+                                        projection=projection)
             result_id.append(doc['anc_id'])
             result_name.append(doc['anc_name'])
         return result_id, result_name
@@ -97,7 +99,7 @@ class QueryTaxonTree(query_nosql.DataQuery):
                 ids: list of organism's ids e.g.[12345, 234456]
 
             Return:
-                result: list of ancestors in order of the farthest to the closest
+                (:obj:`tuple` of :obj:`list`): list of ancestors in order of the farthest to the closest
         '''
         result_name = []
         result_id = []
@@ -127,13 +129,14 @@ class QueryTaxonTree(query_nosql.DataQuery):
                 ancestor: closest common ancestor's name
                 distance: each organism's distance to the ancestor
         '''
+        if org1.upper() == org2.upper():
+            return (org1, [0, 0])
+
         if org_format == 'name':
-            anc_ids, anc_names = self.get_anc_by_name([org1, org2])
+            anc_ids, _ = self.get_anc_by_name([org1, org2])
         else:
-            anc_ids, anc_names = self.get_anc_by_id([org1, org2])
+            anc_ids, _ = self.get_anc_by_id([org1, org2])
 
-        if org1 == org2:
-            return ('org1', [0, 0])
         org1_anc = anc_ids[0]
         org2_anc = anc_ids[1]
 
